@@ -1,5 +1,6 @@
 #!/usr/bin/python -u
-# Copyright (c) 2004 Danilo Segan <danilo@kvota.net>.
+# -*- encoding: utf-8 -*-
+# Copyright (c) 2004, 2005, 2006 Danilo Šegan <danilo@gnome.org>.
 #
 # This file is part of xml2po.
 #
@@ -118,7 +119,7 @@ msgstr ""
 
     def outputAll(self, out):
         self.outputHeader(out)
-        
+
         for k in self.messages:
             if k in self.comments:
                 out.write("#. %s\n" % (self.comments[k].replace("\n","\n#. ")))
@@ -133,6 +134,8 @@ msgstr ""
             if self.do_translations:
                 if len(self.translations)>0:
                     translation = self.translations.pop(0)
+            if translation == k:
+                translation = ""
             out.write("msgstr \"%s\"\n\n" % (translation))
 
 
@@ -192,7 +195,7 @@ def normalizeString(text, ignorewhitespace = 1):
 
     result = re.sub('^ ','', result)
     result = re.sub(' $','', result)
-    
+
     return result
 
 def stringForEntity(node):
@@ -242,14 +245,10 @@ def getTranslation(text, spacepreserve = 0):
     text = normalizeString(text, not spacepreserve)
     if (text.strip() == ''):
         return text
-    file = open(mofile, "rb")
-    if file:
-        myfallback = NoneTranslations()
-        gt = gettext.GNUTranslations(file)
-        gt.add_fallback(myfallback)
-        if gt:
-            res = gt.ugettext(text.decode('utf-8'))
-            return res
+    global gt
+    if gt:
+        res = gt.ugettext(text.decode('utf-8'))
+        return res
 
     return text
 
@@ -287,14 +286,14 @@ def startTagForNode(node):
                     nsprop = p.name
                 params += " %s=\"%s\"" % (nsprop, myAttributeSerialize(p))
     return result+params
-        
+
 def endTagForNode(node):
     if not node:
         return 0
 
     result = node.name
     return result
-        
+
 def isFinalNode(node):
     if automatic:
         auto = autoNodeIsFinal(node)
@@ -354,6 +353,9 @@ def getCommentForNode(node):
     else:
         return None
 
+def replaceAttributeContentsWithText(node,text):
+    node.setContent(text)
+
 def replaceNodeContentsWithText(node,text):
     """Replaces all subnodes of a node with contents of text treated as XML."""
 
@@ -368,7 +370,7 @@ def replaceNodeContentsWithText(node,text):
             tmp = tmp + dtd.serialize('utf-8')
         except libxml2.treeError:
             pass
-            
+
         content = '<%s>%s</%s>' % (starttag, text, endtag)
         tmp = tmp + content.encode('utf-8')
 
@@ -399,6 +401,7 @@ def replaceNodeContentsWithText(node,text):
                 next = node.next
                 node.replaceNode(newelem.copyNodeList())
                 node.next = next
+
         else:
             # In practice, this happens with tags such as "<para>    </para>" (only whitespace in between)
             pass
@@ -407,9 +410,15 @@ def replaceNodeContentsWithText(node,text):
 
 def autoNodeIsFinal(node):
     """Returns 1 if node is text node, contains non-whitespace text nodes or entities."""
-    final = 0
+    if hasattr(node, '__autofinal__'):
+        return node.__autofinal__
+    if node.name in ignored_tags:
+        node.__autofinal__ = 0
+        return 0
     if node.isText() and node.content.strip()!='':
+        node.__autofinal__ = 1
         return 1
+    final = 0
     child = node.children
     while child:
         if child.type in ['text'] and  child.content.strip()!='':
@@ -417,15 +426,20 @@ def autoNodeIsFinal(node):
             break
         child = child.next
 
+    node.__autofinal__ = final
     return final
 
 
-def worthOutputting(node):
+def worthOutputting(node, noauto = 0):
     """Returns 1 if node is "worth outputting", otherwise 0.
 
     Node is "worth outputting", if none of the parents
     isFinalNode, and it contains non-blank text and entities.
     """
+    if noauto and hasattr(node, '__worth__'):
+        return node.__worth__
+    elif not noauto and hasattr(node, '__autoworth__'):
+        return node.__autoworth__
     worth = 1
     parent = node.parent
     final = isFinalNode(node) and node.name not in ignored_tags
@@ -437,13 +451,37 @@ def worthOutputting(node):
             break
         parent = parent.parent
     if not worth:
+        node.__worth__ = 0
         return 0
 
-    return autoNodeIsFinal(node)
-    
+    if noauto:
+        node.__worth__ = worth
+        return worth
+    else:
+        node.__autoworth__ = autoNodeIsFinal(node)
+        return node.__autoworth__
+
+def processAttribute(node, attr):
+    if not node or not attr or not worthOutputting(node=node, noauto=1):
+        return
+
+    outtxt = attr.content
+    if mode=='merge':
+        translation = getTranslation(outtxt, 0)
+        replaceAttributeContentsWithText(attr, translation.encode('utf-8'))
+    else:
+        msg.outputMessage(outtxt, node.lineNo(),  "", 0,
+                          node.name + ":" + attr.name)
+
 def processElementTag(node, replacements, restart = 0):
     """Process node with node.type == 'element'."""
     if node.type == 'element':
+        # Translate attributes if needed
+        if node.properties and len(treated_attributes):
+            for p in node.properties:
+                if p.name in treated_attributes:
+                    processAttribute(node, p)
+
         outtxt = ''
         if restart:
             myrepl = []
@@ -552,7 +590,7 @@ def doSerialize(node):
             child = child.next
         return outtxt
 
-    
+
 def read_finaltags(filelist):
     if CurrentXmlMode:
         return CurrentXmlMode.getFinalTags()
@@ -570,6 +608,13 @@ def read_ignoredtags(filelist):
                     'varlistentry' ]
         return defaults
 
+def read_treatedattributes(filelist):
+    if CurrentXmlMode:
+        return CurrentXmlMode.getTreatedAttributes()
+    else:
+        return []
+
+
 def tryToUpdate(allargs, lang):
     # Remove "-u" and "--update-translation"
     print >>sys.stderr, "OVDI!"
@@ -648,6 +693,7 @@ default_mode = 'docbook'
 filename = ''
 origxml = ''
 mofile = ''
+gt = None
 ultimate = [ ]
 ignored = [ ]
 filenames = [ ]
@@ -764,8 +810,18 @@ if mode=='merge' and mofile=='':
     print >> sys.stderr, "Error: You must specify MO file when merging translations."
     sys.exit(3)
 
+if mofile:
+    try:
+        mfile = open(mofile, "rb")
+
+        gt = gettext.GNUTranslations(mfile)
+        gt.add_fallback(NoneTranslations())
+    except:
+        print >> sys.stderr, "Can't open MO file '%s'." % (mofile)
+
 ultimate_tags = read_finaltags(ultimate)
 ignored_tags = read_ignoredtags(ignored)
+treated_attributes = read_treatedattributes(ignored)
 
 # I'm not particularly happy about making any of these global,
 # but I don't want to bother too much with it right now
